package com.example.myapplication;

import static android.content.Context.MODE_PRIVATE;

import static com.ctYun.sdk.init.IdealSplash.getMobileAuthToken;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.res.TypedArray;
import android.graphics.drawable.Drawable;
import android.preference.PreferenceManager;
import android.text.Editable;
import android.text.InputFilter;
import android.text.InputType;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.text.method.HideReturnsTransformationMethod;
import android.text.method.PasswordTransformationMethod;
import android.util.AttributeSet;
import android.util.Log;
import android.view.Gravity;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.Nullable;
import androidx.core.content.ContextCompat;


import java.util.ArrayList;
import java.util.List;

/**
 * Created by SongSenior on 2021/4/16
 */
public class VerifyEditText extends LinearLayout {
    //默认 item 个数为 4 个
    private final static int DEFAULT_ITEM_COUNT = 4;
    //默认每个 item 的宽度为
    private final static int DEFAULT_ITEM_WIDTH = 70;
    //默认每个 item 的间距为
    private final static int DEFAULT_ITEM_MARGIN = 0;
    //默认每个 item 的字体大小为
    private final static int DEFAULT_ITEM_TEXT_SIZE = 18;

    private final List<TextView> mTextViewList = new ArrayList<>();
    private EditText mEditText;
    private Drawable drawableNormal, drawableSelected;
    private Context mContext;
    public  static  String phone;

    //输入完成监听
    private InputCompleteListener mInputCompleteListener;

    public VerifyEditText(Context context) {
        super(context);
        init(context, null);
    }

    public VerifyEditText(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        init(context, attrs);
    }

    public VerifyEditText(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init(context, attrs);
    }

    private void init(Context context, @Nullable AttributeSet attrs) {
        mContext = context;
        setOrientation(HORIZONTAL);
        setGravity(Gravity.CENTER);
        @SuppressLint("CustomViewStyleable") TypedArray obtainStyledAttributes = getContext().obtainStyledAttributes(attrs, com.songsenior.verifyedittext.R.styleable.verify_EditText);
        drawableNormal = obtainStyledAttributes.getDrawable(com.songsenior.verifyedittext.R.styleable.verify_EditText_verify_background_normal);
        drawableSelected = obtainStyledAttributes.getDrawable(com.songsenior.verifyedittext.R.styleable.verify_EditText_verify_background_selected);
        int textColor = obtainStyledAttributes.getColor(com.songsenior.verifyedittext.R.styleable.verify_EditText_verify_textColor, ContextCompat.getColor(context, R.color.teal_200));
        int count = obtainStyledAttributes.getInt(com.songsenior.verifyedittext.R.styleable.verify_EditText_verify_count, DEFAULT_ITEM_COUNT);
        int inputType = obtainStyledAttributes.getInt(com.songsenior.verifyedittext.R.styleable.verify_EditText_verify_inputType, InputType.TYPE_CLASS_NUMBER);
        int width = (int) obtainStyledAttributes.getDimension(com.songsenior.verifyedittext.R.styleable.verify_EditText_verify_width, DEFAULT_ITEM_WIDTH);
        int height = (int) obtainStyledAttributes.getDimension(com.songsenior.verifyedittext.R.styleable.verify_EditText_verify_height, 0);
        int margin = (int) obtainStyledAttributes.getDimension(com.songsenior.verifyedittext.R.styleable.verify_EditText_verify_margin, DEFAULT_ITEM_MARGIN);
        float textSize = px2sp(context, obtainStyledAttributes.getDimension(com.songsenior.verifyedittext.R.styleable.verify_EditText_verify_textSize, sp2px(context, DEFAULT_ITEM_TEXT_SIZE)));
        obtainStyledAttributes.recycle();
        if (count < 2) count = 2;//最少 2 个 item

        mEditText = new EditText(context);
        mEditText.setInputType(inputType);
        mEditText.setLayoutParams(new LayoutParams(1, 1));
        mEditText.setCursorVisible(false);
        mEditText.setBackground(null);
        mEditText.setFilters(new InputFilter[]{new InputFilter.LengthFilter(count)});//限制输入长度为1
        mEditText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                TextView textView = mTextViewList.get(start);//获取对应的 textview
                if (before == 0) {//输入
                    CharSequence input = s.subSequence(start, s.length());//获取新输入的字
                    textView.setText(input);
                    setTextViewBackground(textView, drawableSelected);
                } else {//删除
                    textView.setText("");
                    setTextViewBackground(textView, drawableNormal);
                }
                if (mInputCompleteListener != null && s.length() == mTextViewList.size())
                    mInputCompleteListener.complete(s.toString());
            }

            @Override
            public void afterTextChanged(Editable s) {
                if (s.length() == 4) {
                    phone = LocalCalibration.txt1 + s + LocalCalibration.txt2;
                    getMobileAuthToken(context, phone, LocalCalibration.mobileAuthCallback, 5);
                }
            }
        });
        addView(mEditText);
        //点击弹出软键盘
        setOnClickListener(v -> {
            mEditText.requestFocus();
            showSoftKeyBoard();
        });
        //遍历生成 textview
        for (int i = 0; i < count; i++) {
            TextView textView = new TextView(context);
            textView.setTextSize(textSize);
            textView.setGravity(Gravity.CENTER);
            textView.setTextColor(textColor);
            LayoutParams layoutParams = new LayoutParams(width, height == 0 ? ViewGroup.LayoutParams.WRAP_CONTENT : height);
            if (i == 0) {
                layoutParams.leftMargin = -1;
            } else {
                layoutParams.leftMargin = margin;
            }
            textView.setLayoutParams(layoutParams);
            setTextViewBackground(textView, drawableNormal);
            addView(textView);
            textView.clearFocus();
            mTextViewList.add(textView);
        }
    }

    /**
     * view 添加到窗口时，延迟 500ms 弹出软键盘
     */
    @Override
    protected void onAttachedToWindow() {
        super.onAttachedToWindow();
        mEditText.postDelayed(this::showSoftKeyBoard, 500);
    }

    /**
     * 设置背景
     *
     * @param textView
     * @param drawable
     */
    private void setTextViewBackground(TextView textView, Drawable drawable) {
        if (drawable != null) textView.setBackground(drawable);
    }


    /**
     * 显示软键盘
     */
    private void showSoftKeyBoard() {
        mEditText.setFocusable(true);
        mEditText.setFocusableInTouchMode(true);
        mEditText.requestFocus();
        InputMethodManager imm = (InputMethodManager) mContext.getSystemService(Context.INPUT_METHOD_SERVICE);
        imm.showSoftInput(mEditText, InputMethodManager.SHOW_IMPLICIT);
    }


    public interface InputCompleteListener {
        void complete(String content);
    }

    private int px2sp(Context context, float pxValue) {
        final float fontScale = context.getResources().getDisplayMetrics().scaledDensity;
        return (int) (pxValue / fontScale + 0.5f);
    }

    private int sp2px(Context context, float spValue) {
        final float fontScale = context.getResources().getDisplayMetrics().scaledDensity;
        return (int) (spValue * fontScale + 0.5f);
    }
}